+++
title = 'Camera recommendations'
date = 2024-07-30T19:12:04Z
draft = true
weight = 8
+++
The following list contains the essential recommendations when selecting a camera to capture high-quality images:  

-  **Frame rate:** Ensure the camera can capture images at a high frame rate, matching the speed of the conveyor belt to avoid motion blurriness. Look for cameras with at least 30 FPS (frames per second) or higher.
-  **Shutter type:** A global shutter is preferable over a rolling shutter to avoid distortion, especially for fast-moving objects.
-  **Lens compatibility:** Choose a camera with interchangeable lenses to adjust the field of view and focus based on the conveyor belt width and object size.
-  **Interface:** Select a camera with an appropriate interface (for example, USB 3.0, GigE, or Camera Link) that supports high-speed data transfer to the processing unit.
-  **Integration with software libraries:** Verify compatibility with software and libraries for seamless integration. For example, if you are using an edge device, such as NVIDIA or Raspberry Pi, you need software integration compatibility with it.
-  **Scan type:** An area scan camera is preferable over a line scan camera to ensure flexibility and comprehensive imaging capabilities required to identify varying waste materials effectively. Area scan cameras are suitable when objects vary significantly in size and shape and for situations where the conveyor belt speed varies or is not uniform.
-  **Resolution:** A suitable resolution is crucial for image analysis because it corresponds to the level of detail, patterns, and textures a camera can detect from an object. You must have an image resolution of at least 1024x1024 pixels. For this reason, choose a camera with a resolution between 0.5 and 1.0 MP (megapixels), depending on the size of the objects and the level of detail required.

{{% panel status="primary" title="Note" icon="far fa-lightbulb" %}}The higher the pixel resolution, the more processing time and latency are introduced by having to process all those pixels through the model. Find a balance between image resolution and processing latency.
{{% /panel %}}

The following list contains additional factors you might want to consider based on your facility's conditions:  
 
-  **Enclosure:** Ensure the camera has an industrial-grade enclosure, preferably NEMA, IP65, or higher, to withstand harsh environments and dust. For example, [Basler has some offerings](https://www.baslerweb.com/en/products/accessories-and-bundles/basler-ip67-housing/).
-  **Lighting:** Ensure diffused or even lighting across all items on the conveyor belt. If the conveyor belt's speed is high, you need a smaller [aperture size](./factors/#aperture-size-f-number) and a higher [shutter speed](./factors/#shutter-speed). The conveyor belt should be well-lit to ensure bright images, reducing blurriness.
-  **Parameter control:** Opt for a camera with good low-light sensitivity and high dynamic range to handle varying lighting conditions. Look for cameras with adjustable exposure settings to accommodate different lighting conditions and object speeds. Also, choose a camera that supports remote configuration for tuning software parameters such as shutter speed, exposure, and frame rate.
-  **Mounting options:** Ensure you have flexible mounting options to position the camera correctly above the conveyor belt. Review different mount accessories for the camera to ensure a safe, fixed, and stable assembly.
-  **Integrated lightning:** Accompany the camera with an integrated lightning system to reach the required luminosity in your facility. You can find accessories such as lightning bulbs or lamps to increase the brightness of the conveyor belt.
-  **Synchronization:** If you need multiple cameras, ensure they can be synchronized to capture images simultaneously.
-  **Power supply:** Consider power over ethernet (PoE).
-  **Temperature range:** Verify the camera can operate within the temperature range of the recycling facility.
-  **Reliability and durability:** Choose cameras with a proven track record for reliability and durability in industrial applications.

## Recommended models

The following list contains some examples of recommended models for your camera:

-  [Arducam High Quality Camera](https://www.arducam.com/product/b0242-arducam-imx477-hq-camera/)
-  [GoPro HERO12 Black](https://gopro.com/en/us/shop/cameras/hero12-black/CHDHX-121-master.html)
