/*!
 * jQuery Plugin Back to the Top
 * Version 1.1.7
 * Update: 2017-05-06 11:14:36
 * Copyright 2017 thingsym
 * URI:
 * Repository: https://github.com/thingsym/jquery.backtothetop
 * License: Dual licensed under the MIT and GPLv2 licenses.
 * http://www.opensource.org/licenses/mit-license.php
 * http://www.gnu.org/licenses/gpl-2.0.html
 */
(function($) {
  $.fn.BackToTheTop = function(options) {
    let settings = $.extend({
      duration: 300,
      easing: 'swing',
      offset: 0,
      hash: true,
      scrolloffset: 0,
      fadein: 'slow',
      fadeout: 'slow',
      display: 'bottom-right',
      top: 0,
      bottom: 0,
      left: 0,
      right: 0,
      zIndex: 999,
      position: 'fixed'
    }, options);

    function scrollToTarget() {
      $('a[href^="#"]').on('click.BackToTheTop', function(event) {
        let targetOffset = $(this).data('backtothetop-scrolltop') !== undefined
          ? $(this).data('backtothetop-scrolltop')
          : $(this.hash).offset() !== undefined ? $(this.hash).offset().top : null;

        if (targetOffset !== null) {
          let duration = $(this).data('backtothetop-duration') !== undefined
            ? $(this).data('backtothetop-duration')
            : settings.duration;

          let easing = $(this).data('backtothetop-easing') !== undefined
            ? $(this).data('backtothetop-easing')
            : settings.easing;

          let offset = $(this).data('backtothetop-offset') !== undefined
            ? $(this).data('backtothetop-offset')
            : settings.offset;

          let hash = $(this).data('backtothetop-hash') !== undefined
            ? $(this).data('backtothetop-hash')
            : settings.hash;

          let href = $(this).attr('href');

          $('html, body').animate({ scrollTop: targetOffset + offset }, duration, easing, function() {
            if (hash) {
              window.history.pushState('', '', href);
            }
          });
          event.preventDefault();
        }
      });
    }

    function handleFixedButton() {
      let fixedButton = $('a#backtothetop-fixed');
      if (fixedButton.length) {
        let scrollOffset = fixedButton.data('backtothetop-fixed-scroll-offset') !== undefined
          ? fixedButton.data('backtothetop-fixed-scroll-offset')
          : settings.scrolloffset;

        let fadeIn = fixedButton.data('backtothetop-fixed-fadein') !== undefined
          ? fixedButton.data('backtothetop-fixed-fadein')
          : settings.fadein;

        let fadeOut = fixedButton.data('backtothetop-fixed-fadeout') !== undefined
          ? fixedButton.data('backtothetop-fixed-fadeout')
          : settings.fadeout;

        let display = fixedButton.data('backtothetop-fixed-display') !== undefined
          ? fixedButton.data('backtothetop-fixed-display')
          : settings.display;

        let top = fixedButton.data('backtothetop-fixed-top') !== undefined
          ? fixedButton.data('backtothetop-fixed-top')
          : settings.top;

        let bottom = fixedButton.data('backtothetop-fixed-bottom') !== undefined
          ? fixedButton.data('backtothetop-fixed-bottom')
          : settings.bottom;

        let left = fixedButton.data('backtothetop-fixed-left') !== undefined
          ? fixedButton.data('backtothetop-fixed-left')
          : settings.left;

        let right = fixedButton.data('backtothetop-fixed-right') !== undefined
          ? fixedButton.data('backtothetop-fixed-right')
          : settings.right;

        let zIndex = fixedButton.data('backtothetop-fixed-zindex') !== undefined
          ? fixedButton.data('backtothetop-fixed-zindex')
          : settings.zIndex;

        if (display === 'top-left') {
          bottom = 'none';
          right = 'none';
        } else if (display === 'top-right') {
          bottom = 'none';
          left = 'none';
        } else if (display === 'bottom-left') {
          top = 'none';
          right = 'none';
        } else if (display === 'bottom-right') {
          top = 'none';
          left = 'none';
        }

        fixedButton.css({ display: 'none' });

        $(window).on('scroll.BackToTheTop', function() {
          if ($(this).scrollTop() > scrollOffset) {
            fixedButton.css({
              top: top,
              bottom: bottom,
              left: left,
              right: right,
              zIndex: zIndex,
              position: settings.position
            });

            if (fixedButton.css('display') === 'none') {
              fixedButton.fadeIn(fadeIn);
            }
          } else if ($(this).scrollTop() <= 0 + scrollOffset && fixedButton.css('display') !== 'none') {
            fixedButton.fadeOut(fadeOut);
          }
        });
      }
    }

    scrollToTarget();
    handleFixedButton();

    return this;
  };

  $(window).on('load.BackToTheTop', function() {
    $('a[href^="#"]').BackToTheTop();
  });

}(jQuery));